%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% An algorithm for cleaning up the coastline, separating the point set
%%%% into a "closed" subset and a "not closed" subset;  Using a stack-type data
%%%% structure;
%%%% Gaoyang Li 06/June/2020 The University of Auckland,
%%%% gli353@aucklanduni.ac.nz
function [MainIsland, IslandsStruct, NewCoastline] = CoastlineProcessing(CoastLineFile)
    %%%% Input the txt file "xxxx.txt" as CoastLineFile
    CoastLine = load(CoastLineFile);
    IslandsIdx = zeros(2,200);  %%% Row1: start point, Row2: end point
    % CurrPosition = CoastLine(1,:);
    % IslandIdx(1,1) = 1;
    N = length(CoastLine(:,1));
    Stack = zeros(N,2);
    Stack(1,:) = CoastLine(1,:);
    counter = 1;
    Islands = zeros(2000,400);
    IslandsCounter = 1;
    flag = 0;
    for i = 2:N
        if flag == 0
            for j = counter : -1 : 1
                if CoastLine(i,1) == Stack(j,1) && CoastLine(i,2) == Stack(j,2)
                    List = [Stack(j:counter,:);CoastLine(i,:)];
                    Stack(j:counter,:) = 0;
                    nn = length(List(:,1));
                    Islands(1:nn,2*IslandsCounter-1:2*IslandsCounter) = List;
                    IslandsIdx(1,IslandsCounter) = i - nn + 1;   IslandsIdx(2,IslandsCounter) = i;
                    IslandsCounter = IslandsCounter + 1;
                    if j > 1
                        if List(end,1) == Stack(j-1,1) && List(end,2) == Stack(j-1,2)
                            Stack(j-1,:) = 0;
                        end
                    end
                    counter = counter - nn + 1;
                    flag = 1;
                    break;
                end
            end
        else
            flag = 0;
        end
        if i+flag <= N
            Stack(counter+1,:) = CoastLine(i+flag,:);
        end
        counter = counter + 1;
    end
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %%%% Prepare approximate x,y coordinates for computing areas of islands
    R = 6371000;

    WestLon = min(CoastLine(:,1));
    EastLon = max(CoastLine(:,1));
    SouthLat = min(CoastLine(:,2));
    NorthLat = max(CoastLine(:,2));
    dlat = NorthLat - SouthLat;
    dlon = EastLon - WestLon;
    a1 = sind(dlat/2)^2 + cosd(NorthLat)*cosd(SouthLat)*sind(0/2)^2;
    c1 = 2*atan2(sqrt(a1),sqrt(1-a1));
    DistY = R*c1;
    a2 = cosd(NorthLat)*cosd(NorthLat)*sind(dlon/2)^2;
    c2 = 2*atan2(sqrt(a2),sqrt(1-a2));
    DistX_north = R*c2;
    a3 = cosd(SouthLat)*cosd(SouthLat)*sind(dlon/2)^2;
    c3 = 2*atan2(sqrt(a3),sqrt(1-a3));
    DistX_south = R*c3;
    area = zeros(1,IslandsCounter - 1);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%% Plot the cleaned coastline
    nfirst = find(Stack(:,1)~=0,1,'first');
    nlast = find(Stack(:,1)~=0,1,'last');
    MainIsland = Stack(nfirst:nlast,:);
    NewCoastline = [flipud(MainIsland);ones(1,2)*NaN];
    plot(MainIsland(:,1),MainIsland(:,2),'k','linewidth',1.5); hold on; 

    MainIsland = NewCoastline;
    plot(MainIsland(:,1),MainIsland(:,2),'k','linewidth',1.5); hold on; 

    for i = 1:IslandsCounter - 1
        L = IslandsIdx(2,i) - IslandsIdx(1,i) + 1;
        plot(Islands(1:L,2*i-1), Islands(1:L,2*i),'r'); hold on;
        if  (Islands(1,2*i-1) ~= Islands(L,2*i-1)) || (Islands(1,2*i) ~= Islands(L,2*i))
            disp('1');
        end
        Coords = [Islands(1:L,2*i-1), Islands(1:L,2*i)];
        NewCoastline = [NewCoastline;Coords;ones(1,2)*NaN];
        area(i) = AreaCalculation(Coords,SouthLat,dlat,WestLon,dlon,DistY,DistX_north,DistX_south);
    end
    %%%%%%%%%%%%%%%% Preapare the struct storing islands
    IslandsStruct.Islands = cell(1,IslandsCounter - 1);
    [~,ind] = sort(area,'descend');
    IslandsStruct.Area = area(ind)/10^6;   %%% in km^2
    for i = 1:IslandsCounter - 1
        L = IslandsIdx(2,ind(i)) - IslandsIdx(1,ind(i)) + 1;
        Coords = [Islands(1:L,2*ind(i)-1), Islands(1:L,2*ind(i))];
        IslandsStruct.Islands{i} = Coords;
    end
end

function area = AreaCalculation(Coords,SouthLat,dlat,WestLon,dlon,DistY,DistX_north,DistX_south)
    Ratio_North = (Coords(:,2) - SouthLat)/dlat;
    Y = Ratio_North * DistY;
    DistX = Ratio_North*DistX_north + (1 - Ratio_North)*DistX_south;
    X = (Coords(:,1) - WestLon)/dlon .* DistX; 
    
    n = length(X);
    area = 0;
    for i = 1:n
        IDXcurr = i;
        IDXnext = i+1;
        if i == n
           IDXnext = 1;
        end
        area = area + 1/2*(X(IDXcurr)*Y(IDXnext) - X(IDXnext)*Y(IDXcurr));
    end
    area = abs(area);    %%% shoelace formula
end